@@ -671,7 +671,8 @@ See also the documentation for `gnus-article-highlight-citation'."
 	      (and (>= (gnus-overlay-end overlay) (point-min))
 		   (<= (gnus-overlay-end overlay) (point-max))))
       (setq gnus-cite-overlay-list (delete overlay gnus-cite-overlay-list))
-      (gnus-delete-overlay overlay))))
+      (ignore-errors
+	(gnus-delete-overlay overlay)))))
 
 (defun gnus-cite-parse-wrapper ()
   ;; Wrap chopped gnus-cite-parse.
